
var sitegin = require('./sitegin/sitegin');

var input = new sitegin.input.Hugo(process.cwd()+'/content/');
var transformerMD = new sitegin.transformer.Markdown;
var transformerTags = new sitegin.transformer.Tags;
var transformerNunjucks = new sitegin.transformer.Nunjucks;
var transformerUrlizeTags = new sitegin.transformer.UrlizeTags;
var output = new sitegin.output.Filesystem;
var sequencer = new sitegin.Sequencer;
var transformerGit = new sitegin.transformer.Git;
var transformerOther = new sitegin.transformer.Other;

var sass = require('node-sass');
var fs = require('fs');

var builddir = "build";
if(process.argv[2] !== "compileonly") builddir = "build-debug";

var oldEmit = sequencer.emit;
sequencer.emit = function() {
  console.log("Event:",arguments[0]);
  oldEmit.apply(sequencer, arguments);
}

sequencer.on("", function(ev) {
  console.log("Event:",ev);
})

var tasks = 0;
var onDone = function() {
  tasks--;
  if(tasks == 0 && process.argv[2] == "compileonly") {
    console.log("exitting");
    process.exit();
  }
}


// PIPELINE
sequencer
.load(input)
.registerNext(transformerGit)
.registerNext(transformerOther)
.registerNext(transformerMD)
.registerNext(transformerUrlizeTags)
.registerNext(transformerTags)
.registerNext(transformerNunjucks)
.registerNext(output)
.finish(function(list) {
  console.log('sequencer done');
  onDone();
})
tasks++;

try {fs.mkdirSync(builddir);}catch(e){}
try {fs.mkdirSync(builddir+"/theme");}catch(e){}

// SASS
var rendersass = function() {
  sass.render ({file: "theme/sass/style.scss"},function(err, result) {
    if(err == null) {
      console.log("compiled sass");
      fs.writeFile(builddir+"/theme/style.css",result.css);
    } else console.log("error ", err);
    console.log("sass done");onDone();
  });
};
tasks++;

rendersass();

// STATIC FILES
var fsextra = require('node-fs-extra');
tasks+=3;
fsextra.copy(
    'static',
    builddir,
    function(file){ return !(file.match(".git") || file.match("static/articles")); },
    function(){console.log("copy static done");onDone();}
  );
fsextra.copy(
    'static/articles',
    builddir+"/clanek",
    function(){console.log("copy static done");onDone();}
  );
fsextra.copy(
    'theme/static',
    builddir,
    function(file){ return !file.match(".git") },
    function(){console.log("copy theme static done");onDone();}
  );

// DEVEL INCOMING
if(process.argv[2] !== "compileonly") {
  var express = require("express");
  var app = express();
  var http = require('http').Server(app);
  var io = require('socket.io')(http);

  // WEBSERVER
  var port = 1337;

  app.use(express.static(builddir)); //use static files in ROOT/builddir folder

  io.on('connection', function(socket){
    console.log('a user connected');
  });

  http.listen(port, function() {
    console.log('listening on *:'+port);
  });

  // CHANGE WATCHER
  var chokidar = require('chokidar');

  chokidar.watch('theme/static', {ignoreInitial: true, usePolling: false})
  .on('all', (event, path) => {
    console.log("theme static copy watcher");
    fsextra.copy(
      'theme/static',
      builddir,
      function(file){ return !file.match(".git"); },
      function(){}
      );
    io.emit('reload');
  });
  
  chokidar.watch('static/', {ignoreInitial: true, usePolling: false})
  .on('all', (event, path) => {
    if(path.match(/^static\/articles\/.*$/)) return;
    console.log("static copy watcher");
    fsextra.copy(
      'static',
      builddir,
      function(file){ return !(file.match(".git") || file.match("static/articles")); },
      function(){}
    );
    io.emit('reload');
  })
  .unwatch('static/articles');
  
  chokidar.watch('static/articles', {ignoreInitial: true, usePolling: false})
  .on('all', (event, path) => {
    console.log("static articles copy watcher");
    fsextra.copy('static/articles',builddir+"/clanek",function(){});
    io.emit('reload');
  })
  .unwatch('static/articles');

  chokidar.watch(['theme/','content/'], {ignoreInitial: true, usePolling: false})
  .on('all', (event, path) => {
    if(path.match(/^theme\/static\/.*$/)) return;
    if(path.match(/^theme\/sass\/.*$/)) return;
    console.log("rebuild watcher",path);
    sequencer.load(input);
  })
  .unwatch("theme/static")
  .unwatch("theme/sass");

  sequencer.finish(function() {
    io.emit('reload');
  });

  chokidar.watch('theme/sass', {ignoreInitial: true, usePolling: false})
  .on('all', function(event, path) {
    console.log("sass: file change detected - rerendering "+event+" "+path);
    rendersass();
    io.emit('reload');
  });
}
